import { world, EntityComponentTypes, BlockComponentTypes } from '@minecraft/server';
import { system } from "@minecraft/server";
import setIcon from "./icons"
let viewDistance = 8;


system.runInterval(
    () => {
        for (const player of world.getAllPlayers()) {

            //Entity Indicator
            let lookingEntities = player.getEntitiesFromViewDirection({
                maxDistance: viewDistance
            });
            if (lookingEntities.length != 0) {

                if (targetIsOnRange(player.getHeadLocation(), lookingEntities[0].entity.location)) {
                    const currentEntity = lookingEntities[0].entity;

                    let itemComponent = currentEntity.getComponent(EntityComponentTypes.Item);

                    let heartsToShow = currentEntity.getComponent(EntityComponentTypes.Health) != null ? getHealthText(currentEntity.getComponent(EntityComponentTypes.Health).currentValue, currentEntity.getComponent(EntityComponentTypes.Health).defaultValue) : ""
                    let isBaby = currentEntity.getComponent(EntityComponentTypes.IsBaby) != null

                    let entityName = currentEntity.typeId.split(":")[1]
                    let modIdentifier = itemComponent != null ? itemComponent.itemStack.typeId.split(":")[0] : currentEntity.typeId.split(":")[0]

                    if (itemComponent == null)
                        player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"entity.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${entityName}.name"}, {"text":"${heartsToShow.length != 0 ? "\n" + heartsToShow : ""}${isBaby ? "\n§7§oBaby" : ""}\n§1§o${modIdentifier}"}]}`)
                    else {
                        const itemName = itemComponent.itemStack.typeId.split(":")[1];
                        player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"item.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${itemName}.name"}, {"text":" x${itemComponent.itemStack.amount}\n§1§o${modIdentifier}"}]}`)
                    }

                }
            } else {

                //If not an entity, then block indicator
                let lookingBlock = player.getBlockFromViewDirection({
                    maxDistance: viewDistance
                });
                if (lookingBlock) {

                    let blockName = lookingBlock.block.typeId.split(":")[1]
                    let modIdentifier = lookingBlock.block.typeId.split(":")[0]

                    let extraData = ``

                    if (targetIsOnRange(player.getHeadLocation(), lookingBlock.block.location)) {

                        const redstonePower = lookingBlock.block.getRedstonePower();

                        if (blockName.includes("log") && modIdentifier === "minecraft" && !blockName.includes("mangrove") && !blockName.includes("cherry")) {
                            let logName = lookingBlock.block.getTags()[2];
                            if (logName == "dark_oak") logName = "big_oak";
                            blockName = "log." + logName;
                        }
                        else if (blockName == "stone")
                            blockName += ".stone"

                        else if (blockName == "seagrass")
                            blockName += ".seagrass"

                        if (player.isSneaking) {
                            extraData += `\nx: ${lookingBlock.block.x} y:${lookingBlock.block.y} z:${lookingBlock.block.z}`

                            if (lookingBlock.block.getComponent(BlockComponentTypes.Sign)) {
                                extraData += "\n" + lookingBlock.block.getComponent(BlockComponentTypes.Sign).getText();
                            }

                            if (redstonePower != undefined) {
                                player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"tile.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${blockName}.name"}, {"text":"\n${getRedstoneText(redstonePower)}"}, {"text":"\n§o§7${extraData}"}, {"text":"\n§1§o${modIdentifier}"}]}`)
                            } else {
                                player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"tile.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${blockName}.name"}, {"text":"\n§o§7${extraData}"}, {"text":"\n§1§o${modIdentifier}"}]}`)
                            }

                        } else {
                            if (redstonePower != undefined) {
                                player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"tile.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${blockName}.name"}, {"text":"\n${getRedstoneText(redstonePower)}"}, {"text":"\n§1§o${modIdentifier}"}]}`)
                            } else {
                                player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"tile.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${blockName}.name"}, {"text":"\n§1§o${modIdentifier}"}]}`)
                            }

                        }
                    }
                }
            }
        }
    }, 1
);



function execute(player) {

    //var player = world.getAllPlayers()[0];


}

function targetIsOnRange(headLocation, targetLocation) {
    return Math.abs(targetLocation.x - headLocation.x) < viewDistance && Math.abs(targetLocation.z - headLocation.z) < viewDistance && Math.abs(targetLocation.y - headLocation.y) < viewDistance
}

function getRedstoneText(amount) {
    let template = `:::redstone${amount}`;
    return setIcon(template, 'redstone', 0xe304);
}

function getHealthText(currentHealth, maxHealth) {
    currentHealth = currentHealth - 1;
    let heartsToShow = ""
    if (maxHealth <= 20) {
        for (let i = 0; i < maxHealth; i++) {
            if (i % 2 == 0) {
                if (i < currentHealth) {
                    heartsToShow += ":::heart";
                } else if (i > currentHealth) {
                    heartsToShow += ":::hempty";
                }
            } else {
                if (i == currentHealth + 1) {
                    heartsToShow += ":::hhalf"
                }
            }
        }
        heartsToShow = setIcon(heartsToShow, 'heart', 0xe300)
        heartsToShow = setIcon(heartsToShow, 'hhalf', 0xe301)
        heartsToShow = setIcon(heartsToShow, 'hempty', 0xe302)
    } else {
        heartsToShow = "§7§oHealth: " + (currentHealth + 1) + " / " + maxHealth
    }

    return heartsToShow
}